<?php
namespace Ak\Woo\Model\Entity\Order;

use Ak\Woo\Model\Entity\Order;
use Ak\Woo\Model\OrderItemInterface;
use Ak\Wp\Model\Entity\MetaAwareEntity;
use Ak\Wp\Model\MetaInterface;
use Doctrine\ORM\Mapping as ORM;

/**
 * Class OrderItem
 *
 * @package Ak\Woo\Model\Entity
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/woo-model
 *
 * @ORM\Entity()
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorColumn(name="order_item_type", type="string")
 * @ORM\Table(name="wp_woocommerce_order_items", options={"charset": "utf8mb4", "collate": "utf8mb4_general_ci"})
 */
abstract class Item extends MetaAwareEntity implements OrderItemInterface
{
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     * @ORM\Column(name="order_item_id", type="bigint", options={"unsigned": true})
     */
    protected ?int $id = null;

    /**
     * @var Order
     *
     * @ORM\ManyToOne(targetEntity="Ak\Woo\Model\Entity\Order", inversedBy="items")
     * @ORM\JoinColumn(name="order_id", referencedColumnName="ID")
     */
    protected Order $order;

    /**
     * @var string
     *
     * @ORM\Column(name="order_item_name", type="string")
     */
    protected string $name = '';

    /**
     * @var MetaInterface[]
     *
     * @ORM\OneToMany(targetEntity="Ak\Woo\Model\Entity\Order\ItemMeta", mappedBy="orderItem")
     */
    protected $metas;

    /**
     * Vraća naziv stavke.
     *
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @param string $key
     * @return MetaInterface
     */
    protected function createMeta(string $key): MetaInterface
    {
        return new ItemMeta($this, $key);
    }
}
